
/* Copyright 2000 Johannes Lundberg, 2:206/149, <jojo@chaosdev.org> */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "../config.h"
#include "fbutil.h"

#define PROGRAM_VERSION "0.0.11"
#define PROGRAM_AUTHOR "Johannes Lundberg"
#define PROGRAM_CONTACT "2:206/149@fidonet, <jojo@chaosdev.org>"
#define PROGRAM_NAME "fbutil"

void usage (void)
{
  printf ("%s %s by %s, %s\n", PROGRAM_NAME, PROGRAM_VERSION, PROGRAM_AUTHOR,
          PROGRAM_CONTACT);
  printf ("Compiled: %s, %s with gcc-%s\n", __TIME__, __DATE__, __VERSION__);
  printf ("\

usage:	fbutil [command] [options]

add area [area]
	Add a file area to the File Database.
	You will be asked for some settings for the new area.

list [ areas | area <area> ]
	List file areas or files in a file area. The area to list can be
	specified by path or by area number.

import <file> [area]
	Import a file into the File Data Base.
	If no area is specified, the program will try to autodetect which
	area the file should be put in.

move <file> [area] [area]
	Move a file between areas. The areas can also be specified as pathes.

delete [ <file> [area] | area <area> ]
	Delete a file from an area, or delete a file area.

");
}

void log (int priority, const char *format, ...)
{
  char *message;
  va_list va_ptr;

  /* Maximum size of text is now 2KB */

  message = malloc (2048);

  va_start (va_ptr, format);
  vsprintf (message, format, va_ptr);
  va_end (va_ptr);

  switch (priority)
  {  
  default:
  case '?':
  case ' ':
    printf ("%c %s", priority, message);
  case '+':		
  case '-':
    Syslog (priority, message);
    break;
  }
  
  free (message);
}


void list_file_area (long area_nr)
{
  FILE *file;
  struct FILERecord filedata;
  struct fileareas areadata;
  int records, counter;

  fbutil_FileAreasInit ();

  if (!FileAreaGetData (&areadata, area_nr))
  {
    printf ("Area %ld - %s - %s\n", area_nr, areadata.Name, areadata.Path);
    printf ("\n");
  }
  else
  {
    log ('?', "Area %ld does not exist?\n", area_nr);
    ExitClient(1);
  }

  if ((file = OpenFileBase (area_nr, FALSE)) == 0)
  {
    log ('?', "File area defined, but the File Database for this area seems to be corrupt\n");
    ExitClient(1);
  }

  fseek (file, 0, SEEK_END);
  records = ftell (file) / sizeof (struct FILERecord);
  rewind (file);

  for (counter = 0; counter < records; counter++)
  {
    fread (&filedata, 1, sizeof (struct FILERecord), file);

    /* Display the first line of the discription for each file. */
    
    printf ("%s\t%s\n", filedata.Name, filedata.Desc[0]);
  }
  
  fclose (file);
}

void view_file (unsigned long area_nr, char *filename)
{
  FILE *file;
  struct FILERecord filedata;
  struct fileareas areadata;
  int records, counter, index;

  fbutil_FileAreasInit ();

  if (!FileAreaGetData (&areadata, area_nr))
  {
    printf ("Area %ld - %s - %s\n", area_nr, areadata.Name, areadata.Path);
    printf ("\n");
  }
  else
  {
    log ('?', "Area %ld does not exist?\n", area_nr);
    ExitClient(1);
  }

  if ((file = OpenFileBase (area_nr, FALSE)) == 0)
  {
    log ('?', "File area defined, but the File Database for this area seems to be corrupt\n");
    ExitClient(1);
  }

  fseek (file, 0, SEEK_END);
  records = ftell (file) / sizeof (struct FILERecord);
  rewind (file);

  for (counter = 0; counter < records; counter++)
  {
    fread (&filedata, 1, sizeof (struct FILERecord), file);

    if (strcmp (filedata.Name, filename) == 0)
    {
      /* We found the file. Now display the file data in a nice way. */

      printf ("File: \t%s\n", filedata.Name);
      printf ("Size: \t%ld KB\n", filedata.Size);
//      printf ("Date: \t%ld\n", filedata.FileDate);
      printf ("Downloaded: %ld times\n", filedata.TimesDL);
      printf ("Uploader: %s\n", filedata.Uploader);
//      printf ("Uploaded: %ld\n", filedata.UploadDate);
      printf ("\n");

      /* The description. */

      for (index = 0; index < 20; index++)
      {
        if (filedata.Desc[index][0] == 0)
        {
          index = 20;
        }
        else
        {
          printf ("%s\n", filedata.Desc[index]);
        }
      }
      
      counter = records + 1;
    }
  }
  
  fclose (file);
}

void add_file_area (unsigned long area_nr)
{
  struct fileareas areadata;
  unsigned long template_area;
  char tmpstr[20];

  fbutil_FileAreasInit ();

  do
  {
    printf ("Template area: ");
    GetstrC (tmpstr, 20);
    template_area = fbutil_resolve_file_area (tmpstr);
    FileAreaGetData (&areadata, template_area);
    printf ("Use area %ld (%s) as template? [y/N] ", template_area, areadata.Name);
  } while (tolower(getchar()) != 'y');

  /* Ask for some infomation from the user. */
  
  do
  {
    printf ("Name: ");
    GetstrC (areadata.Name, sizeof (areadata.Name));
    printf ("Path: ");
    GetstrC (areadata.Path, sizeof (areadata.Path));
    printf ("---\nName: %s\nPath: %s\nIs this correct? [Y/n] \n",
            areadata.Name, areadata.Path);
  } while (tolower (getchar ()) == 'n');

  AddFileArea (areadata, area_nr);
}

void delete_file_area (unsigned long area_nr)
{
  struct fileareas areadata;

  fbutil_FileAreasInit ();
  FileAreaGetData (&areadata, area_nr);

  printf ("Do you want to delete area %ld? (%s) [y/N] ", area_nr, areadata.Name);

  if (tolower (getchar ()) == 'y')
  {
    log ('!', "** Deleting area %ld.\n", area_nr);
    DeleteFileArea (area_nr);
  }
}

void file_import (char *filename, unsigned long area_nr)
{
  struct stat statfile;

  if (stat(filename, &statfile))
  {
    log ('!', "File %s does not exist.\n", filename);
    ExitClient(1);
  }

  fbutil_FileAreasInit ();

  printf ("Adding %s to area %ld.\n", filename, area_nr);

  Addfile (filename, area_nr, TRUE);
}

void file_move (char *name, long from, long to)
{
  struct fileareas fromdata, todata;
  char filename[160], destination[160];
  struct stat statfile;
  FILE *fromfdb, *tofdb;
  long records;
  struct FILERecord filedata;
  int counter;
  char command[256];

  fbutil_FileAreasInit ();

  printf ("Attemping to move file %s from area %ld to %ld.\n", name, from, to);

  FileAreaGetData (&fromdata, from);
  FileAreaGetData (&todata, to);
  printf ("(%s in %s -> %s)\n", name, fromdata.Name, todata.Name);

  sprintf (filename, "%s/%s", fromdata.Path, name);
  if (stat(filename, &statfile))
  {
    log ('!', "File %s does not exist.\n", filename);
    ExitClient(1);
  }

  fromfdb = OpenFileBase (from, TRUE);

  fseek (fromfdb, 0, SEEK_END);
  records = ftell (fromfdb) / sizeof (struct FILERecord);
  rewind (fromfdb);

  for (counter = 0; counter < records; counter++)
  {
    fread (&filedata, 1, sizeof (struct FILERecord), fromfdb);

    if (!strcmp (filedata.Name, name))
    {
      printf ("Found data for %s in record %d.\n", name, counter);
      counter = records;
      
      printf ("-> %s\n", filedata.Desc[0]);
    
      /* Set delete flag on the old file. */
    
      filedata.Deleted = TRUE;
    
      fseek (fromfdb, -(sizeof (struct FILERecord)), SEEK_CUR);
      fwrite (&filedata, 1, sizeof (struct FILERecord), fromfdb);
    }
  }

  fclose (fromfdb);

  tofdb = OpenFileBase (to, TRUE);
  
  fseek (tofdb, 0, SEEK_END);

  fwrite (&filedata, 1, sizeof (struct FILERecord), tofdb);

  fclose (fromfdb);
  
  /* Move the actual file. */
  
  sprintf (destination, "%s/%s", todata.Path, name);
  
  printf ("Move file %s to %s? [Y/n] ", filename, destination);

  if (toupper (getchar ()) != 'N')
  {
    printf ("\n  Moving...");
    fflush (stdout);
    sprintf (command, "mv %s %s 2>&1 > /dev/null", filename, destination);
    if (system (command) == 0)
    {
      printf ("done.\n");
    }
    else
    {
      printf ("failed.\n");
    }
  }
  else
  {
    printf ("\n");
  }
}

/* Primary uses for testing new functions and so. */

#ifdef DEVEL
void test (void)
{
}
#endif

/* The main function just parses the arguments and passes the control over
   to the right function. */

void parse_arguments (int arguments, char *argument[])
{
  unsigned long area_nr, from, to;

  if (arguments < 2)
  {
    usage ();
    ExitClient(1);
  }  
#ifdef DEVEL
  else if (!strcmp (argument[1], "test"))
  {
    test ();
  }
#endif
  else if (arguments < 3)
  {
    usage ();
    ExitClient(1);
  }
  else if (!strcmp (argument[1], "list") && !strcmp (argument[2], "areas"))
  {
  }
  else if (!strcmp (argument[1], "view") && arguments == 4)
  {
    area_nr = fbutil_resolve_file_area (argument[3]);
    
    view_file (area_nr, argument[2]);
  }
  else if (!strcmp (argument[1], "list") && !strcmp (argument[2], "area") &&
           arguments == 4)
  {
    fbutil_FileAreasInit ();
    area_nr = fbutil_resolve_file_area (argument[3]);
    list_file_area (area_nr);
  }
  else if (!strcmp (argument[1], "add") && !strcmp (argument[2], "area") &&
           arguments == 4)
  {
    if ((area_nr = atol (argument[3])) <= 0)
    {
      usage ();
      ExitClient(1);
    }

    add_file_area (area_nr);
  }
  else if (!strcmp (argument[1], "delete") && !strcmp (argument[2], "area") &&
           arguments == 4)
  {
    if ((area_nr = atol (argument[3])) <= 0)
    {
      usage ();
      ExitClient(1);
    }

    delete_file_area (area_nr);
  }
  else if (!strcmp (argument[1], "import"))
  {
    if (arguments == 4)
    {
      area_nr = fbutil_resolve_file_area (argument[3]);

      file_import (argument[2], area_nr);
    }
    else if (arguments == 3)
    {
      /* Try to figure out in to which file area the file should be
         imported. */

      fbutil_FileAreasInit ();

      if ((area_nr = FileAreaGetByPath (getenv ("PWD"))))
      {
        file_import (argument[2], area_nr);
      }
      else
      {
        log ('!', "Autodetection of filearea failed. Please specify.\n");
        ExitClient(1);
      }      
    }
    else
    {
      log ('!', "File to import not specified!\n\n");
      ExitClient(1);
    }
  }
  else if ((!strcmp (argument[1], "delete")) && (arguments == 4))
  {
    fbutil_FileAreasInit ();

    area_nr = fbutil_resolve_file_area (argument[3]);

    FileDelete (argument[2], area_nr);
  }
  else if ((!strcmp (argument[1], "move")) && (arguments == 5))
  {
    fbutil_FileAreasInit ();

    from = fbutil_resolve_file_area (argument[3]);

    to = fbutil_resolve_file_area (argument[4]);

    file_move (argument[2], from, to);
  }
  else
  {
    usage ();
    ExitClient(1);
  }
}

int main (int arguments, char *argument[])
{
  struct passwd *pw;
#ifdef MEMWATCH
        mwInit();
#endif

  InitConfig ();
  TermInit (1);

  pw = getpwuid (getuid());

  InitClient (pw->pw_name, (char *)"fbutil", CFG.location, CFG.logfile,
             CFG.util_loglevel, CFG.error_log);

  Syslog (' ', "");
  Syslog (' ', "%s %s by %s, %s\n", PROGRAM_NAME, PROGRAM_VERSION,
          PROGRAM_AUTHOR, PROGRAM_CONTACT);

  parse_arguments (arguments, argument);
 
  ExitClient(0); 
  return 0;
}
